import re

import aexpect
from avocado.utils import process

from virttest import utils_misc
from virttest import error_context


class QemuIOParamError(Exception):

    """
    Parameter Error for qemu-io command
    """
    pass


class QemuIO(object):

    """
    A class for execute qemu-io command
    """

    def __init__(self, test, params, image_name, blkdebug_cfg="",
                 prompt=r"qemu-io>\s*$", log_filename=None, io_options="",
                 log_func=None):
        self.type = ""
        if log_filename:
            log_filename += "-" + utils_misc.generate_random_string(4)
            self.output_func = utils_misc.log_line
            self.output_params = (log_filename,)
        else:
            self.output_func = None
            self.output_params = ()
        self.output_prefix = ""
        self.prompt = prompt
        self.blkdebug_cfg = blkdebug_cfg

        self.qemu_io_cmd = utils_misc.get_qemu_io_binary(params)
        self.io_options = io_options
        self.run_command = False
        self.image_name = image_name
        self.blkdebug_cfg = blkdebug_cfg
        self.log_func = log_func

    def get_cmd_line(self, ignore_option=[], essential_option=[],
                     forbid_option=[]):
        """
        Generate the command line for qemu-io from the parameters
        :params ignore_option: list for the options should not in command
        :params essential_option: list for the essential options
        :params forbid_option: list for the option should not in command
        :return: qemu-io command line
        """
        essential_flag = False

        qemu_io_cmd = self.qemu_io_cmd
        if self.io_options:
            for io_option in re.split(",", self.io_options):
                if io_option in ignore_option:
                    pass
                elif io_option in forbid_option:
                    raise QemuIOParamError
                else:
                    if not essential_flag and io_option in essential_option:
                        essential_flag = True
                    if len(io_option) == 1:
                        qemu_io_cmd += " -%s" % io_option
                    else:
                        qemu_io_cmd += " --%s" % io_option
            if essential_option and not essential_flag:
                raise QemuIOParamError

        if self.image_name:
            qemu_io_cmd += " "
            if self.blkdebug_cfg:
                qemu_io_cmd += "blkdebug:%s:" % self.blkdebug_cfg
            qemu_io_cmd += self.image_name

        return qemu_io_cmd

    def cmd_output(self, command):
        """
        Run a command in qemu-io
        """
        pass

    def close(self):
        """
        Clean up
        """
        pass


class QemuIOShellSession(QemuIO):

    """
    Use a shell session to execute qemu-io command
    """

    def __init__(self, test, params, image_name, blkdebug_cfg="",
                 prompt=r"qemu+-io>\s*$", log_filename=None, io_options="",
                 log_func=None):
        QemuIO.__init__(self, test, params, image_name, blkdebug_cfg, prompt,
                        log_filename, io_options, log_func)

        self.type = "shell"
        forbid_option = ["h", "help", "V", "version", "c", "cmd"]

        self.qemu_io_cmd = self.get_cmd_line(forbid_option=forbid_option)
        self.create_session = True
        self.session = None

    @error_context.context_aware
    def cmd_output(self, command, timeout=60):
        """
        Get output from shell session. If the create flag is True, init the
        shell session and set the create flag to False.
        :param command: command to execute in qemu-io
        :param timeout: timeout for execute the command
        """
        qemu_io_cmd = self.qemu_io_cmd
        prompt = self.prompt
        output_func = self.output_func
        output_params = self.output_params
        output_prefix = self.output_prefix
        if self.create_session:
            error_context.context(
                "Running command: %s" % qemu_io_cmd, self.log_func)
            self.session = aexpect.ShellSession(qemu_io_cmd, echo=True,
                                                prompt=prompt,
                                                output_func=output_func,
                                                output_params=output_params,
                                                output_prefix=output_prefix)
            # Record the command line in log file
            if self.output_func:
                params = self.output_params + (qemu_io_cmd, )
                self.output_func(*params)

            self.create_session = False
            # Get the reaction from session
            self.session.cmd_output("\n")

        error_context.context("Executing command: %s" % command, self.log_func)
        return self.session.cmd_output(command, timeout=timeout)

    def close(self):
        """
        Close the shell session for qemu-io
        """
        if not self.create_session:
            self.session.close()


class QemuIOSystem(QemuIO):

    """
    Run qemu-io with a command line which will return immediately
    """

    def __init__(self, test, params, image_name, blkdebug_cfg="",
                 prompt=r"qemu-io>\s*$", log_filename=None, io_options="",
                 log_func=None):
        QemuIO.__init__(self, test, params, image_name, blkdebug_cfg, prompt,
                        log_filename, io_options, log_func)
        ignore_option = ["c", "cmd"]
        essential_option = ["h", "help", "V", "version", "c", "cmd"]

        self.qemu_io_cmd = self.get_cmd_line(ignore_option=ignore_option,
                                             essential_option=essential_option)

    @error_context.context_aware
    def cmd_output(self, command, timeout=60):
        """
        Get output from system_output. Add the command to the qemu-io command
        line with -c and record the output in the log file.
        :param command: command to execute in qemu-io
        :param timeout: timeout for execute the command
        """
        qemu_io_cmd = self.qemu_io_cmd
        if command:
            qemu_io_cmd += " -c '%s'" % command

        error_context.context(
            "Running command: %s" % qemu_io_cmd, self.log_func)
        output = process.run(qemu_io_cmd, timeout=timeout).stdout_text

        # Record command line in log file
        if self.output_func:
            params = self.output_params + (qemu_io_cmd,)
            self.output_func(*params)

            params = self.output_params + (output,)
            self.output_func(*params)

        return output

    def close(self):
        """
        To keep the the same interface with QemuIOShellSession
        """
        pass



"""
Module of common exceptions used in libvirt_xml package
"""


class LibvirtXMLError(Exception):

    """
    Error originating within libvirt_xml module
    """

    def __init__(self, details=''):
        self.details = details
        Exception.__init__(self)

    def __str__(self):
        return str(self.details)


class LibvirtXMLAccessorError(LibvirtXMLError):

    """
    LibvirtXMLError related to an accessor generator class/method
    """
    pass


class LibvirtXMLForbiddenError(LibvirtXMLError):

    """
    LibvirtXMLError raised when operating on a property is prohibited
    """
    pass


class LibvirtXMLNotFoundError(LibvirtXMLError):

    """
    LibvirtXMLError related when an element cannot be found
    """
    pass



"""
vlan protocl support class(es)

http://libvirt.org/formatnwfilter.html#nwfelemsRulesProtoVLAN
"""

from virttest.libvirt_xml import accessors, xcepts
from virttest.libvirt_xml.nwfilter_protocols import base


class Vlan(base.TypedDeviceBase):

    """
    Create new Vlan xml instances

    Properties:
        attrs: libvirt_xml.nwfilter_protocols.Vlan.Attr instance
    """

    __slots__ = ('attrs',)

    def __init__(self, type_name='file', virsh_instance=base.base.virsh):
        accessors.XMLElementNest('attrs', self, parent_xpath='/',
                                 tag_name='vlan', subclass=self.Attr,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        super(Vlan, self).__init__(protocol_tag='vlan', type_name=type_name,
                                   virsh_instance=virsh_instance)

    def new_attr(self, **dargs):
        """
        Return a new Attr instance and set properties from dargs

        :param dargs: dict of attributes
        :return: new Attr instance
        """
        new_one = self.Attr(virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one

    def get_attr(self):
        """
        Return vlan attribute dict

        :return: None if no vlan in xml, dict of vlan's attributes.
        """
        try:
            vlan_node = self.xmltreefile.reroot('/vlan')
        except KeyError as detail:
            raise xcepts.LibvirtXMLError(detail)
        node = vlan_node.getroot()
        vlan_attr = dict(list(node.items()))

        return vlan_attr

    class Attr(base.base.LibvirtXMLBase):

        """
        Vlan attribute XML class

        Properties:

        srcmacaddr: string, MAC address of sender
        srcmacmask: string, Mask applied to MAC address of sender
        dstmacaddr: string, MAC address of destination
        dstmacaddr: string, Mask applied to MAC address of destination
        vlanid: string, VLAN ID
        encap_protocol: string, Encapsulated layer 3 protocol ID
        comment: string, text with max. 256 characters
        """

        __slots__ = ('srcmacaddr', 'srcmacmask', 'dstmacaddr', 'dstmacmask',
                     'vlanid', 'encap_protocol', 'comment')

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLAttribute('srcmacaddr', self, parent_xpath='/',
                                   tag_name='vlan', attribute='srcmacaddr')
            accessors.XMLAttribute('srcmacmask', self, parent_xpath='/',
                                   tag_name='vlan', attribute='srcmacmask')
            accessors.XMLAttribute('dstmacaddr', self, parent_xpath='/',
                                   tag_name='vlan', attribute='dstmacaddr')
            accessors.XMLAttribute('dstmacmask', self, parent_xpath='/',
                                   tag_name='vlan', attribute='dstmacmask')
            accessors.XMLAttribute('vlanid', self, parent_xpath='/',
                                   tag_name='vlan', attribute='vlanid')
            accessors.XMLAttribute('encap_protocol', self, parent_xpath='/',
                                   tag_name='vlan', attribute='encap-protocol')
            accessors.XMLAttribute('comment', self, parent_xpath='/',
                                   tag_name='vlan', attribute='comment')

            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<vlan/>'



import os

from avocado.utils import path as utils_path

from . import data_dir
from . import cartesian_config
from .compat import get_opt

GUEST_NAME_LIST = None
TAG_INDEX = {}


def _variant_only_file(filename):
    """
    Parse file containing flat list of items to append on an 'only' filter
    """
    if not os.path.isabs(filename):
        filename = os.path.realpath(os.path.join(data_dir.get_root_dir(),
                                                 filename))
    return ", ".join([_.strip() for _ in open(filename)
                      if not _.lstrip().startswith('#')])


#: The first entry of these lists will be used as a default value.
#: Only change the first entry if you intend to change the default
#: value.  See :func:`avocado_vt.plugins.vt.add_basic_vt_options` and
#: class:`avocado_vt.options.VirtTestOptionsProcess` for usage examples.
SUPPORTED_TEST_TYPES = [
    'qemu', 'libvirt', 'libguestfs', 'openvswitch', 'v2v', 'lvsb', 'spice']

SUPPORTED_LIBVIRT_URIS = ['qemu:///system', 'lxc:///']
SUPPORTED_LIBVIRT_DRIVERS = ['qemu', 'lxc', 'xen']

SUPPORTED_IMAGE_TYPES = ['qcow2', 'raw', 'qed', 'vmdk']
SUPPORTED_DISK_BUSES = ['virtio_blk', 'ide', 'scsi',
                        'virtio_scsi', 'lsi_scsi', 'ahci', 'usb2', 'xenblk']
SUPPORTED_NIC_MODELS = ["virtio_net", "e1000", "rtl8139", "spapr-vlan"]
SUPPORTED_NET_TYPES = ["bridge", "user", "none"]


def find_default_qemu_paths(options_qemu=None, options_dst_qemu=None):
    if options_qemu:
        if not os.path.isfile(options_qemu):
            raise RuntimeError("Invalid qemu binary provided (%s)" %
                               options_qemu)
        qemu_bin_path = options_qemu
    else:
        try:
            qemu_bin_path = utils_path.find_command('qemu-kvm')
        except utils_path.CmdNotFoundError:
            qemu_bin_path = utils_path.find_command('kvm')

    if options_dst_qemu is not None:
        if not os.path.isfile(options_dst_qemu):
            raise RuntimeError("Invalid dst qemu binary provided (%s)" %
                               options_dst_qemu)
        qemu_dst_bin_path = options_dst_qemu
    else:
        qemu_dst_bin_path = None

    qemu_dirname = os.path.dirname(qemu_bin_path)
    qemu_img_path = os.path.join(qemu_dirname, 'qemu-img')
    qemu_io_path = os.path.join(qemu_dirname, 'qemu-io')

    if not os.path.exists(qemu_img_path):
        qemu_img_path = utils_path.find_command('qemu-img')

    if not os.path.exists(qemu_io_path):
        qemu_io_path = utils_path.find_command('qemu-io')

    return [qemu_bin_path, qemu_img_path, qemu_io_path, qemu_dst_bin_path]


def get_cartesian_parser_details(cartesian_parser):
    """
    Print detailed information about filters applied to the cartesian cfg.

    :param cartesian_parser: Cartesian parser object.
    """
    details = ""
    details += ("Tests produced by config file %s\n\n" %
                cartesian_parser.filename)

    details += "The full test list was modified by the following:\n\n"

    if cartesian_parser.only_filters:
        details += "Filters applied:\n"
        for flt in cartesian_parser.only_filters:
            details += "    %s\n" % flt

    if cartesian_parser.no_filters:
        for flt in cartesian_parser.no_filters:
            details += "    %s\n" % flt

    details += "\n"
    details += "Different guest OS have different test lists\n"
    details += "\n"

    if cartesian_parser.assignments:
        details += "Assignments applied:\n"
        for flt in cartesian_parser.assignments:
            details += "    %s\n" % flt

    details += "\n"
    details += "Assignments override values previously set in the config file\n"
    details += "\n"

    return details


def get_guest_name_parser(options):
    cartesian_parser = cartesian_config.Parser()
    machines_cfg_path = data_dir.get_backend_cfg_path(get_opt(options, 'vt.type'),
                                                      'machines.cfg')
    guest_os_cfg_path = data_dir.get_backend_cfg_path(get_opt(options, 'vt.type'),
                                                      'guest-os.cfg')
    cartesian_parser.parse_file(machines_cfg_path)
    cartesian_parser.parse_file(guest_os_cfg_path)
    if get_opt(options, 'vt.common.arch'):
        cartesian_parser.only_filter(get_opt(options, 'vt.common.arch'))
    if get_opt(options, 'vt.common.machine_type'):
        cartesian_parser.only_filter(get_opt(options, 'vt.common.machine_type'))
    if get_opt(options, 'vt.guest_os'):
        cartesian_parser.only_filter(get_opt(options, 'vt.guest_os'))
    return cartesian_parser


def get_guest_name_list(options):
    global GUEST_NAME_LIST
    if GUEST_NAME_LIST is None:
        guest_name_list = []
        for params in get_guest_name_parser(options).get_dicts():
            shortname = ".".join(params['name'].split(".")[1:-1])
            guest_name_list.append(shortname)

        GUEST_NAME_LIST = guest_name_list

    return GUEST_NAME_LIST



"""
seclabel device support class(es)

http://libvirt.org/formatdomain.html#seclabel
"""

from virttest.libvirt_xml import accessors
from virttest.libvirt_xml.devices import base


class Seclabel(base.TypedDeviceBase):

    """
    Seclabel XML class

    Properties:
        model:
            string, security driver model
        relabel:
            string, 'yes' or 'no'
        baselabel:
            string, base label string
        label:
            string, the sec label string
    """

    __slots__ = ('model', 'relabel', 'baselabel', 'label')

    def __init__(self, type_name='dynamic', virsh_instance=base.base.virsh):
        accessors.XMLAttribute('model', self, parent_xpath='/',
                               tag_name='seclabel', attribute='model')
        accessors.XMLAttribute('relabel', self, parent_xpath='/',
                               tag_name='seclabel', attribute='relabel')
        accessors.XMLElementText('baselabel', self, parent_xpath='/',
                                 tag_name='baselabel')
        accessors.XMLElementText('label', self, parent_xpath='/',
                                 tag_name='label')
        super(Seclabel, self).__init__(device_tag='seclabel',
                                       type_name=type_name,
                                       virsh_instance=virsh_instance)
        self.xml = '<seclabel></seclabel>'



"""
Helpers for export one image by qemu-nbd,which can be consumed by VM.
qemu-img create -f qcow2 /tmp/disk.qcow2 1G
disk xml:
    <disk type='network' device='disk'>
      <driver name='qemu' type='qcow2'/>
      <source protocol='nbd' tls='no'>
        <host name='localhost' port='10001'/>
      </source>
      <target dev='sda' bus='scsi'/>
    </disk>
"""
import logging
import os
import shutil

from avocado.utils import process

from virttest import utils_config
from virttest import utils_libvirtd
from virttest import utils_net

from virttest.utils_conn import build_server_key, build_CA, build_client_key


class NbdExport(object):
    """qemu-nbd export disk images.

    use it by referring to below steps:
    1) nbd = NbdExport("/var/lib/libvirt/images/nbd.qcow2", port=nbd_server_port, export_name=export_name, tls=tls_enabled)
    2) nbd.start_nbd_server()
       ....
    3) nbd.cleanup()
    """

    def __init__(self, image, image_format="raw", image_size="1G", port="10001", export_name=None, tls=False, deleteExisted=True,
                 private_key_encrypt_passphrase=None, secret_uuid=None):
        """Create a new NbdExport instance

        :param image: instance of image created
        :param image_format: image format,e.g raw or qcom2
        :param image_size: image size
        :param port: port where qemu-nbd export image
        :param export_name: exported alias name
        :param tls: whether use tls for encryption communication
        :param deleteExisted: whether delete currently existed image file
        :param private_key_encrypt_passphrase: password which is used to encrypt private key
        :param secret_uuid: secret id that need written into qemu.conf
        """
        # Input
        self.image = image
        self.image_format = image_format
        self.image_size = image_size
        self.port = port
        self.export_name = export_name
        self.tls = tls
        if self.tls:
            self.qemu_conf = None
            self.libvirtd = None
            self.private_key_encrypt_passphrase = private_key_encrypt_passphrase
            self.secret_uuid = secret_uuid
        self.deleteExisted = deleteExisted

    def _create_img(self):
        """Create a image file with specified format"""
        if os.path.exists(self.image):
            logging.info('image already existed')
            if self.deleteExisted:
                os.remove(self.image)
            else:
                return
        logging.debug("create one image .... if not existed")
        process.run("qemu-img create" + ' -f %s %s %s' % (self.image_format, self.image, self.image_size),
                    ignore_status=True, shell=True, verbose=True)

    def update_nbd_tls_config(self):
        """update nbd tls config in qemu.conf"""
        self.qemu_conf = utils_config.LibvirtQemuConfig()
        client_cert_dir = "/etc/pki/libvirt-nbd"
        self.qemu_conf.nbd_tls_x509_cert_dir = "%s" % client_cert_dir
        self.qemu_conf.nbd_tls = True
        if self.secret_uuid:
            self.qemu_conf.nbd_tls_x509_secret_uuid = "%s" % self.secret_uuid
        self.libvirtd = utils_libvirtd.Libvirtd()
        self.libvirtd.restart()

    def setup_certs(self):
        """setup CA and certs"""
        tmp_ca_cert_dir = '/etc/pki/qemu'
        # Remove old CA information.
        if os.path.exists(tmp_ca_cert_dir):
            shutil.rmtree(tmp_ca_cert_dir)

        # Get server hostname.
        hostname = process.run('hostname', ignore_status=False, shell=True, verbose=True).stdout_text.strip()
        server_ip = utils_net.get_host_ip_address()
        # Initialize the CA information.
        cn = hostname
        ca_credential_dict = {}
        ca_credential_dict['cakey'] = 'ca-key.pem'
        ca_credential_dict['cacert'] = 'ca-cert.pem'
        if not os.path.exists(tmp_ca_cert_dir):
            os.makedirs(tmp_ca_cert_dir)
        build_CA(tmp_ca_cert_dir, cn, certtool="certtool",
                 credential_dict=ca_credential_dict)

        # Create CA folder
        ca_folder = '/etc/pki/CA'
        ca_folder_private = '/etc/pki/CA/private'
        if os.path.exists(ca_folder):
            shutil.rmtree(ca_folder)
        if not os.path.exists(ca_folder):
            os.makedirs(ca_folder)
        if not os.path.exists(ca_folder_private):
            os.makedirs(ca_folder_private)
        # Copy ca-key.pem and ca-cert.pem into CA folder.
        shutil.copy2(os.path.join(tmp_ca_cert_dir, ca_credential_dict['cakey']), os.path.join(ca_folder_private, ca_credential_dict['cakey']))

        # Create server certs and keys
        server_cert_dir = '/etc/pki/qemu'
        if not os.path.exists(server_cert_dir):
            os.makedirs(server_cert_dir)
        server_credential_dict = {}
        server_credential_dict['cakey'] = 'ca-key.pem'
        server_credential_dict['cacert'] = 'ca-cert.pem'
        server_credential_dict['serverkey'] = 'server-key.pem'
        server_credential_dict['servercert'] = 'server-cert.pem'
        server_credential_dict['ca_cakey_path'] = tmp_ca_cert_dir
        # Build a server key.

        build_server_key(server_cert_dir, cn, server_ip, certtool="certtool",
                         credential_dict=server_credential_dict, on_local=True)

        server_key_path = os.path.join(server_cert_dir, server_credential_dict['serverkey'])

        # Create server certs and keys
        client_cert_dir = '/etc/pki/libvirt-nbd'
        # Clean up old client certs and keys.
        if os.path.exists(client_cert_dir):
            shutil.rmtree(client_cert_dir)
        if not os.path.exists(client_cert_dir):
            os.makedirs(client_cert_dir)
        # Copy ca-key.pem and ca-cert.pem into client_cert_dir
        shutil.copy2(os.path.join(tmp_ca_cert_dir, ca_credential_dict['cakey']), os.path.join(client_cert_dir, ca_credential_dict['cakey']))
        shutil.copy2(os.path.join(tmp_ca_cert_dir, ca_credential_dict['cacert']), os.path.join(client_cert_dir, ca_credential_dict['cacert']))
        client_credential_dict = {}
        client_credential_dict['cakey'] = 'ca-key.pem'
        client_credential_dict['cacert'] = 'ca-cert.pem'
        client_credential_dict['clientkey'] = 'client-key.pem'
        client_credential_dict['clientcert'] = 'client-cert.pem'
        if self.private_key_encrypt_passphrase:
            client_credential_dict['clientprivatekeypass'] = self.private_key_encrypt_passphrase
        server_credential_dict['ca_cakey_path'] = tmp_ca_cert_dir

        # build a client key.
        build_client_key(client_cert_dir,
                         client_cn=cn, certtool="certtool",
                         credential_dict=client_credential_dict)

    def start_nbd_server(self):
        """start nbd server"""
        # Clean up pre-occupied port if existed.
        pre_clean = "kill -9 $(ps aux | grep 'qemu-nbd'|grep %s | awk '{print $2}')" % self.port
        process.run(pre_clean, ignore_status=True, shell=True, verbose=False)
        # Create image.
        self._create_img()
        # Start qemu nbd server.
        try:
            tls_object_str = ' '
            if self.tls:
                self.setup_certs()
                self.update_nbd_tls_config()
                tls_creds = 'libivrt_%s' % self.port
                tls_object_str = "--object tls-creds-x509,id=%s,endpoint=server,dir=/etc/pki/qemu  --tls-creds=%s" % (tls_creds, tls_creds)

            qemu_nbd_cmd = 'qemu-nbd -t %s -f %s %s -p %s ' % (tls_object_str, self.image_format, self.image, self.port)
            if self.export_name:
                qemu_nbd_cmd += "-x %s " % self.export_name
            qemu_nbd_cmd += "&"
            process.run(qemu_nbd_cmd, ignore_status=False, shell=True, verbose=True, ignore_bg_processes=True)
            logging.info("nbd server start at port: %s", self.port)
        except Exception as info:
            logging.debug("nbd server fail to start")
            raise

    def stop_nbd_server(self):
        """stop nbd server"""
        kill_cmd = "kill -9 $(ps aux | grep 'qemu-nbd'|grep %s | awk '{print $2}')" % self.port
        process.run(kill_cmd, ignore_status=True, shell=True, verbose=False)

    def cleanTLS(self):
        """clean TLS"""
        logging.debug("enter cleanup TLS now...")
        if self.tls:
            ca_folder = '/etc/pki/CA'
            if os.path.exists(ca_folder):
                shutil.rmtree(ca_folder)
            server_cert_dir = '/etc/pki/qemu'
            if os.path.exists(server_cert_dir):
                shutil.rmtree(server_cert_dir)
            client_cert_dir = '/etc/pki/libvirt-nbd'
            if os.path.exists(client_cert_dir):
                shutil.rmtree(client_cert_dir)
        if self.qemu_conf:
            self.qemu_conf.restore()
        if self.libvirtd:
            self.libvirtd.restart()

    def cleanup(self):
        """clean up environments"""
        self.stop_nbd_server()
        if os.path.exists(self.image) and self.deleteExisted:
            os.remove(self.image)
        if self.tls:
            self.cleanTLS()



import sys
import os


if len(sys.argv) != 3:
    print("Useage: %s path size")

path = sys.argv[1]
size = int(sys.argv[2])

if not os.path.isdir(os.path.dirname(path)):
    os.mkdir(os.path.dirname(path))
writefile = open(path, 'w')
writefile.seek(1024 * 1024 * size)
writefile.write('\x00')
writefile.close()



import logging
import multiprocessing
import os
import shutil
import tarfile

from avocado.core import exceptions
from avocado.utils import download
from avocado.utils import git
from avocado.utils import path
from avocado.utils import process

from virttest import data_dir


def _force_copy(src, dest):
    """
    Replace dest with a new copy of src, even if it exists
    """
    if os.path.isfile(dest):
        os.remove(dest)
    if os.path.isdir(dest):
        dest = os.path.join(dest, os.path.basename(src))
    shutil.copyfile(src, dest)
    return dest


class GitRepoParamHelper(git.GitRepoHelper):

    """
    Helps to deal with git repos specified in cartersian config files

    This class attempts to make it simple to manage a git repo, by using a
    naming standard that follows this basic syntax:

    <prefix>_name_<suffix>

    <prefix> is always 'git_repo' and <suffix> sets options for this git repo.
    Example for repo named foo:

    git_repo_foo_uri = git://git.foo.org/foo.git
    git_repo_foo_base_uri = /home/user/code/foo
    git_repo_foo_branch = master
    git_repo_foo_lbranch = master
    git_repo_foo_commit = bb5fb8e678aabe286e74c4f2993dc2a9e550b627
    """

    def __init__(self, params, name, destination_dir):
        """
        Instantiates a new GitRepoParamHelper
        """
        self.params = params
        self.name = name
        self.destination_dir = destination_dir
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to this repo

        This method currently does everything that the parent class __init__()
        method does, that is, sets all instance variables needed by other
        methods. That means it's not strictly necessary to call parent's
        __init__().
        """
        config_prefix = 'git_repo_%s' % self.name
        logging.debug('Parsing parameters for git repo %s, configuration '
                      'prefix is %s' % (self.name, config_prefix))

        self.base_uri = self.params.get('%s_base_uri' % config_prefix)
        if self.base_uri is None:
            logging.debug('Git repo %s base uri is not set' % self.name)
        else:
            logging.debug('Git repo %s base uri: %s' % (self.name,
                                                        self.base_uri))

        self.uri = self.params.get('%s_uri' % config_prefix)
        logging.debug('Git repo %s uri: %s' % (self.name, self.uri))

        self.branch = self.params.get('%s_branch' % config_prefix, 'master')
        logging.debug('Git repo %s branch: %s' % (self.name, self.branch))

        self.lbranch = self.params.get('%s_lbranch' % config_prefix)
        if self.lbranch is None:
            self.lbranch = self.branch
        logging.debug('Git repo %s lbranch: %s' % (self.name, self.lbranch))

        self.commit = self.params.get('%s_commit' % config_prefix)
        if self.commit is None:
            logging.debug('Git repo %s commit is not set' % self.name)
        else:
            logging.debug('Git repo %s commit: %s' % (self.name, self.commit))

        self.tag = self.params.get('%s_tag' % config_prefix)
        if self.tag is None:
            logging.debug('Git repo %s tag is not set' % self.name)
        else:
            logging.debug('Git repo %s tag: %s' % (self.name, self.tag))

        self.key_file = None
        tag_signed = self.params.get('%s_tag_signed' % config_prefix)
        if tag_signed is None:
            logging.warning('Git repo %s tag is not signed' % self.name)
            logging.warning('This means we will not verify if the key was '
                            'made by whomever claims to have made it '
                            '(dangerous)')
        else:
            self.key_file = os.path.join(data_dir.get_data_dir(), 'gpg',
                                         tag_signed)
            if os.path.isfile(self.key_file):
                logging.debug('Git repo %s tag %s will be verified with public '
                              'key file %s', self.name, self.tag, self.key_file)
            else:
                raise exceptions.TestError('GPG public key file %s not found, '
                                           'will not proceed with testing' %
                                           self.key_file)

        self.cmd = path.find_command('git')

        self.recursive = self.params.get('%s_recursive', 'yes')

    def execute(self):
        super(GitRepoParamHelper, self).execute()

        cwd = os.path.curdir
        os.chdir(self.destination_dir)
        process.system('git remote add origin %s' %
                       self.uri, ignore_status=True)
        if self.recursive == 'yes':
            process.system('git submodule init')
            process.system('git submodule update')

        if self.tag:
            process.system('git checkout %s' % self.tag)
            if self.key_file is not None:
                try:
                    gnupg_home = os.path.join(data_dir.get_tmp_dir(),
                                              'gnupg')
                    if not os.path.isdir(gnupg_home):
                        os.makedirs(gnupg_home)
                    os.environ['GNUPGHOME'] = gnupg_home
                    process.system('gpg --import %s' % self.key_file)
                    logging.debug('Verifying if tag is actually signed with '
                                  'GPG key ID %s' % self.key_file)
                    process.system('git tag -v %s' % self.tag)
                except process.CmdError:
                    raise exceptions.TestError("GPG signature check for git repo "
                                               "%s failed" % self.name)

        # Log the top commit message, good for quick reference
        process.system('git log -1')

        os.chdir(cwd)


class LocalSourceDirHelper(object):

    """
    Helper class to deal with source code sitting somewhere in the filesystem
    """

    def __init__(self, source_dir, destination_dir):
        """
        :param source_dir:
        :param destination_dir:
        :return: new LocalSourceDirHelper instance
        """
        self.source = source_dir
        self.destination = destination_dir

    def execute(self):
        """
        Copies the source directory to the destination directory
        """
        if os.path.isdir(self.destination):
            shutil.rmtree(self.destination)

        if os.path.isdir(self.source):
            shutil.copytree(self.source, self.destination)


class LocalSourceDirParamHelper(LocalSourceDirHelper):

    """
    Helps to deal with source dirs specified in cartersian config files

    This class attempts to make it simple to manage a source dir, by using a
    naming standard that follows this basic syntax:

    <prefix>_name_<suffix>

    <prefix> is always 'local_src' and <suffix> sets options for this source
    dir.  Example for source dir named foo:

    local_src_foo_path = /home/user/foo
    """

    def __init__(self, params, name, destination_dir):
        """
        Instantiate a new LocalSourceDirParamHelper
        """
        self.params = params
        self.name = name
        self.destination_dir = destination_dir
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to source dir
        """
        config_prefix = 'local_src_%s' % self.name
        logging.debug('Parsing parameters for local source %s, configuration '
                      'prefix is %s' % (self.name, config_prefix))

        self.path = self.params.get('%s_path' % config_prefix)
        logging.debug('Local source directory %s path: %s' % (self.name,
                                                              self.path))
        self.source = self.path
        self.destination = self.destination_dir


class LocalTarHelper(object):

    """
    Helper class to deal with source code in a local tarball
    """

    def __init__(self, source, destination_dir):
        self.source = source
        self.destination = destination_dir

    def extract(self):
        """
        Extracts the tarball into the destination directory
        """
        if os.path.isdir(self.destination):
            shutil.rmtree(self.destination)

        if os.path.isfile(self.source) and tarfile.is_tarfile(self.source):

            name = os.path.basename(self.destination)
            temp_dir = os.path.join(os.path.dirname(self.destination),
                                    '%s.tmp' % name)
            logging.debug('Temporary directory for extracting tarball is %s' %
                          temp_dir)

            if not os.path.isdir(temp_dir):
                os.makedirs(temp_dir)

            tarball = tarfile.open(self.source)
            tarball.extractall(temp_dir)

            #
            # If there's a directory at the toplevel of the tarfile, assume
            # it's the root for the contents, usually source code
            #
            tarball_info = tarball.members[0]
            if tarball_info.isdir():
                content_path = os.path.join(temp_dir,
                                            tarball_info.name)
            else:
                content_path = temp_dir

            #
            # Now move the content directory to the final destination
            #
            shutil.move(content_path, self.destination)

        else:
            raise OSError("%s is not a file or tar file" % self.source)

    def execute(self):
        """
        Executes all action this helper is supposed to perform

        This is the main entry point method for this class, and all other
        helper classes.
        """
        self.extract()


class LocalTarParamHelper(LocalTarHelper):

    """
    Helps to deal with source tarballs specified in cartersian config files

    This class attempts to make it simple to manage a tarball with source code,
    by using a  naming standard that follows this basic syntax:

    <prefix>_name_<suffix>

    <prefix> is always 'local_tar' and <suffix> sets options for this source
    tarball.  Example for source tarball named foo:

    local_tar_foo_path = /tmp/foo-1.0.tar.gz
    """

    def __init__(self, params, name, destination_dir):
        """
        Instantiates a new LocalTarParamHelper
        """
        self.params = params
        self.name = name
        self.destination_dir = destination_dir
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to this local tar helper
        """
        config_prefix = 'local_tar_%s' % self.name
        logging.debug('Parsing parameters for local tar %s, configuration '
                      'prefix is %s' % (self.name, config_prefix))

        self.path = self.params.get('%s_path' % config_prefix)
        logging.debug('Local source tar %s path: %s' % (self.name,
                                                        self.path))
        self.source = self.path
        self.destination = self.destination_dir


class RemoteTarHelper(LocalTarHelper):

    """
    Helper that fetches a tarball and extracts it locally
    """

    def __init__(self, source_uri, destination_dir):
        self.source = source_uri
        self.destination = destination_dir

    def execute(self):
        """
        Executes all action this helper class is supposed to perform

        This is the main entry point method for this class, and all other
        helper classes.

        This implementation fetches the remote tar file and then extracts
        it using the functionality present in the parent class.
        """
        name = os.path.basename(self.source)
        base_dest = os.path.dirname(self.destination)
        dest = os.path.join(base_dest, name)
        download.get_file(self.source, dest)
        self.source = dest
        self.extract()


class RemoteTarParamHelper(RemoteTarHelper):

    """
    Helps to deal with remote source tarballs specified in cartersian config

    This class attempts to make it simple to manage a tarball with source code,
    by using a  naming standard that follows this basic syntax:

    <prefix>_name_<suffix>

    <prefix> is always 'local_tar' and <suffix> sets options for this source
    tarball.  Example for source tarball named foo:

    remote_tar_foo_uri = http://foo.org/foo-1.0.tar.gz
    """

    def __init__(self, params, name, destination_dir):
        """
        Instantiates a new RemoteTarParamHelper instance
        """
        self.params = params
        self.name = name
        self.destination_dir = destination_dir
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to this remote tar helper
        """
        config_prefix = 'remote_tar_%s' % self.name
        logging.debug('Parsing parameters for remote tar %s, configuration '
                      'prefix is %s' % (self.name, config_prefix))

        self.uri = self.params.get('%s_uri' % config_prefix)
        logging.debug('Remote source tar %s uri: %s' % (self.name,
                                                        self.uri))
        self.source = self.uri
        self.destination = self.destination_dir


class PatchHelper(object):

    """
    Helper that encapsulates the patching of source code with patch files
    """

    def __init__(self, source_dir, patches):
        """
        Initializes a new PatchHelper
        """
        self.source_dir = source_dir
        self.patches = patches

    def download(self):
        """
        Copies patch files from remote locations to the source directory
        """
        for patch in self.patches:
            download.get_file(patch, os.path.join(self.source_dir,
                                                  os.path.basename(patch)))

    def patch(self):
        """
        Patches the source dir with all patch files
        """
        os.chdir(self.source_dir)
        for patch in self.patches:
            process.system('patch -p1 < %s' % os.path.basename(patch),
                           shell=True)

    def execute(self):
        """
        Performs all steps necessary to download patches and apply them
        """
        self.download()
        self.patch()


class PatchParamHelper(PatchHelper):

    """
    Helps to deal with patches specified in cartersian config files

    This class attempts to make it simple to patch source coude, by using a
    naming standard that follows this basic syntax:

    [<git_repo>|<local_src>|<local_tar>|<remote_tar>]_<name>_patches

    <prefix> is either a 'local_src' or 'git_repo', that, together with <name>
    specify a directory containing source code to receive the patches. That is,
    for source code coming from git repo foo, patches would be specified as:

    git_repo_foo_patches = ['http://foo/bar.patch', 'http://foo/baz.patch']

    And for for patches to be applied on local source code named also foo:

    local_src_foo_patches = ['http://foo/bar.patch', 'http://foo/baz.patch']
    """

    def __init__(self, params, prefix, source_dir):
        """
        Initializes a new PatchParamHelper instance
        """
        self.params = params
        self.prefix = prefix
        self.source_dir = source_dir
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to this set of patches

        This method currently does everything that the parent class __init__()
        method does, that is, sets all instance variables needed by other
        methods. That means it's not strictly necessary to call parent's
        __init__().
        """
        logging.debug('Parsing patch parameters for prefix %s' % self.prefix)
        patches_param_key = '%s_patches' % self.prefix

        self.patches_str = self.params.get(patches_param_key, '[]')
        logging.debug('Patches config for prefix %s: %s' % (self.prefix,
                                                            self.patches_str))

        self.patches = eval(self.patches_str)
        logging.debug('Patches for prefix %s: %s' % (self.prefix,
                                                     ", ".join(self.patches)))


class GnuSourceBuildInvalidSource(Exception):

    """
    Exception raised when build source dir/file is not valid
    """
    pass


class SourceBuildFailed(Exception):

    """
    Exception raised when building with parallel jobs fails

    This serves as feedback for code using
    :class:`virttest.build_helper.BuildHelper`.
    """
    pass


class SourceBuildParallelFailed(Exception):

    """
    Exception raised when building with parallel jobs fails

    This serves as feedback for code using
    :class:`virttest.build_helper.BuildHelper`.
    """
    pass


class GnuSourceBuildHelper(object):

    """
    Handles software installation of GNU-like source code

    This basically means that the build will go though the classic GNU
    autotools steps: ./configure, make, make install
    """

    def __init__(self, source, build_dir, prefix,
                 configure_options=[]):
        """
        :type source: string
        :param source: source directory or tarball
        :type prefix: string
        :param prefix: installation prefix
        :type build_dir: string
        :param build_dir: temporary directory used for building the source code
        :type configure_options: builtin.list
        :param configure_options: options to pass to configure
        :throws: GnuSourceBuildInvalidSource
        """
        self.source = source
        self.build_dir = build_dir
        self.prefix = prefix
        self.configure_options = configure_options
        self.install_debug_info = True
        self.include_pkg_config_path()

    def include_pkg_config_path(self):
        """
        Adds the current prefix to the list of paths that pkg-config searches

        This is currently not optional as there is no observed adverse side
        effects of enabling this. As the "prefix" is usually only valid during
        a test run, we believe that having other pkg-config files (``*.pc``) in
        either ``<prefix>/share/pkgconfig`` or ``<prefix>/lib/pkgconfig`` is
        exactly for the purpose of using them.

        :return: None
        """
        env_var = 'PKG_CONFIG_PATH'

        include_paths = [os.path.join(self.prefix, 'share', 'pkgconfig'),
                         os.path.join(self.prefix, 'lib', 'pkgconfig')]

        if env_var in os.environ:
            paths = os.environ[env_var].split(':')
            for include_path in include_paths:
                if include_path not in paths:
                    paths.append(include_path)
            os.environ[env_var] = ':'.join(paths)
        else:
            os.environ[env_var] = ':'.join(include_paths)

        logging.debug('PKG_CONFIG_PATH is: %s' % os.environ['PKG_CONFIG_PATH'])

    def get_configure_path(self):
        """
        Checks if 'configure' exists, if not, return 'autogen.sh' as a fallback
        """
        configure_path = os.path.abspath(os.path.join(self.source,
                                                      "configure"))
        autogen_path = os.path.abspath(os.path.join(self.source,
                                                    "autogen.sh"))
        if os.path.exists(configure_path):
            return configure_path
        elif os.path.exists(autogen_path):
            return autogen_path
        else:
            raise GnuSourceBuildInvalidSource(
                'configure script does not exist')

    def get_available_configure_options(self):
        """
        Return the list of available options of a GNU like configure script

        This will run the "configure" script at the source directory

        :return: list of options accepted by configure script
        """
        help_raw = process.run('%s --help' % self.get_configure_path(),
                               ignore_status=True).stdout_text
        help_output = help_raw.split("\n")
        option_list = []
        for line in help_output:
            cleaned_line = line.lstrip()
            if cleaned_line.startswith("--"):
                option = cleaned_line.split()[0]
                option = option.split("=")[0]
                option_list.append(option)

        return option_list

    def enable_debug_symbols(self):
        """
        Enables option that leaves debug symbols on compiled software

        This makes debugging a lot easier.
        """
        enable_debug_option = "--disable-strip"
        if enable_debug_option in self.get_available_configure_options():
            self.configure_options.append(enable_debug_option)
            logging.debug('Enabling debug symbols with option: %s' %
                          enable_debug_option)

    def get_configure_command(self):
        """
        Formats configure script with all options set

        :return: string with all configure options, including prefix
        """
        prefix_option = "--prefix=%s" % self.prefix
        options = self.configure_options
        options.append(prefix_option)
        return "%s %s" % (self.get_configure_path(),
                          " ".join(options))

    def configure(self):
        """
        Runs the "configure" script passing appropriate command line options
        """
        configure_command = self.get_configure_command()
        logging.info('Running configure on build dir')
        os.chdir(self.build_dir)
        process.system(configure_command)

    def make_parallel(self):
        """
        Runs "make" using the correct number of parallel jobs
        """
        parallel_make_jobs = multiprocessing.cpu_count()
        make_command = "make -j %s" % parallel_make_jobs
        logging.info("Running parallel make on build dir")
        os.chdir(self.build_dir)
        process.system(make_command)

    def make_non_parallel(self):
        """
        Runs "make", using a single job
        """
        os.chdir(self.build_dir)
        process.system("make")

    def make_clean(self):
        """
        Runs "make clean"
        """
        os.chdir(self.build_dir)
        process.system("make clean")

    def make(self, failure_feedback=True):
        """
        Runs a parallel make, falling back to a single job in failure

        :param failure_feedback: return information on build failure by raising
                                 the appropriate exceptions
        :raise: SourceBuildParallelFailed if parallel build fails, or
                SourceBuildFailed if single job build fails
        """
        try:
            self.make_parallel()
        except process.CmdError:
            try:
                self.make_clean()
                self.make_non_parallel()
            except process.CmdError:
                if failure_feedback:
                    raise SourceBuildFailed
            if failure_feedback:
                raise SourceBuildParallelFailed

    def make_install(self):
        """
        Runs "make install"
        """
        os.chdir(self.build_dir)
        process.system("make install")

    install = make_install

    def make_rpm(self):
        """
        Run "make rpm"
        """
        os.chdir(self.build_dir)
        process.system("make rpm")

    package = make_rpm

    def execute(self):
        """
        Runs appropriate steps for *building* this source code tree
        """
        if self.install_debug_info:
            self.enable_debug_symbols()
        self.configure()
        self.make()


class LinuxKernelBuildHelper(object):

    """
    Handles Building Linux Kernel.
    """

    def __init__(self, params, prefix, source):
        """
        :type params: dict
        :param params: dictionary containing the test parameters
        :type source: string
        :param source: source directory or tarball
        :type prefix: string
        :param prefix: installation prefix
        """
        self.params = params
        self.prefix = prefix
        self.source = source
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to guest kernel
        """
        configure_opt_key = '%s_config' % self.prefix
        self.config = self.params.get(configure_opt_key, '')

        build_image_key = '%s_build_image' % self.prefix
        self.build_image = self.params.get(build_image_key,
                                           'arch/x86/boot/bzImage')

        build_target_key = '%s_build_target' % self.prefix
        self.build_target = self.params.get(build_target_key, 'bzImage')

        kernel_path_key = '%s_kernel_path' % self.prefix
        image_dir = os.path.join(data_dir.get_data_dir(), 'images')
        default_kernel_path = os.path.join(image_dir,
                                           self.build_target)
        self.kernel_path = self.params.get(kernel_path_key,
                                           default_kernel_path)

        logging.info('Parsing Linux kernel build parameters for %s',
                     self.prefix)

    def make_guest_kernel(self):
        """
        Runs "make", using a single job
        """
        os.chdir(self.source)
        logging.info("Building guest kernel")
        logging.debug("Kernel config is %s" % self.config)
        download.get_file(self.config, '.config')

        # FIXME currently no support for builddir
        # run old config
        process.system('yes "" | make oldconfig > /dev/null', shell=True)
        parallel_make_jobs = multiprocessing.cpu_count()
        make_command = "make -j %s %s" % (
            parallel_make_jobs, self.build_target)
        logging.info("Running parallel make on src dir")
        process.system(make_command)

    def make_clean(self):
        """
        Runs "make clean"
        """
        os.chdir(self.source)
        process.system("make clean")

    def make(self, failure_feedback=True):
        """
        Runs a parallel make

        :param failure_feedback: return information on build failure by raising
                                 the appropriate exceptions
        :raise: SourceBuildParallelFailed if parallel build fails, or
        """
        try:
            self.make_clean()
            self.make_guest_kernel()
        except process.CmdError:
            if failure_feedback:
                raise SourceBuildParallelFailed

    def cp_linux_kernel(self):
        """
        Copying Linux kernel to target path
        """
        os.chdir(self.source)
        _force_copy(self.build_image, self.kernel_path)

    install = cp_linux_kernel

    def execute(self):
        """
        Runs appropriate steps for *building* this source code tree
        """
        self.make()


class GnuSourceBuildParamHelper(GnuSourceBuildHelper):

    """
    Helps to deal with gnu_autotools build helper in cartersian config files

    This class attempts to make it simple to build source coude, by using a
    naming standard that follows this basic syntax:

    [<git_repo>|<local_src>]_<name>_<option> = value

    To pass extra options to the configure script, while building foo from a
    git repo, set the following variable:

    git_repo_foo_configure_options = --enable-feature
    """

    def __init__(self, params, name, destination_dir, install_prefix):
        """
        Instantiates a new GnuSourceBuildParamHelper
        """
        self.params = params
        self.name = name
        self.destination_dir = destination_dir
        self.install_prefix = install_prefix
        self._parse_params()

    def _parse_params(self):
        """
        Parses the params items for entries related to source directory

        This method currently does everything that the parent class __init__()
        method does, that is, sets all instance variables needed by other
        methods. That means it's not strictly necessary to call parent's
        __init__().
        """
        logging.debug('Parsing gnu_autotools build parameters for %s' %
                      self.name)

        configure_opt_key = '%s_configure_options' % self.name
        configure_options = self.params.get(configure_opt_key, '').split()
        logging.debug('Configure options for %s: %s' % (self.name,
                                                        configure_options))

        self.source = self.destination_dir
        self.build_dir = self.destination_dir
        self.prefix = self.install_prefix
        self.configure_options = configure_options
        self.include_pkg_config_path()

        # Support the install_debug_info feature, that automatically
        # adds/keeps debug information on generated libraries/binaries
        install_debug_info_cfg = self.params.get("install_debug_info", "yes")
        self.install_debug_info = install_debug_info_cfg != "no"



import sys
try:
    from types import ClassType
except ImportError:
    Classtype = type


"""
Versioning system provides class hierarchy which automatically select the right
version of a class. Class and module manipulation is used for this reason.

By this reason is:
    Advantage) Only one class with some version is working in one process.
               It is possible use class variables. Others version of same
               class have different class variables. Supports pickling.
               Much cleaner debugging and faster running of code than
               using of __getattribute__.
    Disadvantage) It is necessary create class with
                        fatory(orig_class, params for is_right_ver)
               access to specific class through class Manager.


Example of usage (in versionable_class_unittest):


# SIMPLE EXAPMLE

from versionable_class import Manager, factory, VersionableClass
#register module to class manager. Necessary for pickling.
man = Manager(__name__)
# pylint: disable=E1003

class VM(object):
    @classmethod
    def _is_right_ver(cls, version):
        return version < 1

    def __init__(self, *args, **kargs):
        super(VM, self).__init__(*args, **kargs)

    def fn1(self):
        print("fn1_VM")



class VM1(VM):
    @classmethod
    def _is_right_ver(cls, version):
        return version >= 1

    def __init__(self, *args, **kargs):
        super(VM1, self).__init__(*args, **kargs)

    def fn1(self):
        print("fn1_VM1")

class VM_container(test_vers.VersionableClass):
    __master__ = VM1


o = test_vers.factory(VM_container, version=0) # return class.
o = o()    # create instance of class
p = test_vers.factory(VM_container, version=2)()
o.fn1()
p.fn1()


# ADVANCED EXAPMLE

from versionable_class import Manager, factory, VersionableClass
man = Manager(__name__)
# pylint: disable=E1003


def qemu_verison():
    return 2


class VM(object):
    __slot__ = ["cls"]

    test_class_vm1 = None

    @classmethod
    def _is_right_ver(cls, *args, **kargs):
        ver = None
        if "qemu_version" in kargs:
            ver = kargs['qemu_version']
        else:
            ver = qemu_verison()
        if ver < 1:
            return True
        else:
            return False


    def __new__(cls, *args, **kargs):
        return super(VM, cls).__new__(cls, *args, **kargs)


    def __init__(self, *args, **kargs):
        super(VM, self).__init__()
        self.cls = self.__class__.__name__


    def __str__(self):
        return "%s" % self.cls


    def func1(self):
        print("VM_func1")


    def func3(self):
        pass


class VM1(VM):
    __slot__ = ["VM1_cls"]
    @classmethod
    def _is_right_ver(cls, *args, **kargs):
        ver = None
        if "qemu_version" in kargs:
            ver = kargs['qemu_version']
        else:
            ver = qemu_verison()
        if ver > 1:
            return True
        else:
            return False

    def __init__(self, *args, **kargs):
        super(VM1, self).__init__(*args, **kargs)
        self.cls = self.__class__.__name__
        self.VM1_cls = "VM1"


    def __str__(self):
        return "%s" % self.cls


    def func1(self):
        super(VM1, self).func1()


    def func2(self):
        print("func2")


    def func3(self):
        pass


class VM_container(VersionableClass):
    __master__ = VM1

    def __new__(cls, *args, **kargs):
        return super(man[cls, VM_container], cls).__new__(cls, *args, **kargs)


class BB(VM_container):
    test_class_bb = None

    def __new__(cls, *args, **kargs):
        return super(man[cls, BB], cls).__new__(cls, *args, **kargs)


    def func1(self):
        super(man[self.__class__, BB], self).func1()


    def func2(self):
        super(man[self.__class__, BB], self).func2()
"""


def isclass(obj):
    """
    :param obj: Object for inspection if obj is class.
    :return: true if the object is a class.
    """
    return isinstance(obj, (type, ClassType))


class ModuleWrapper(object):

    """
    Wrapper around module.

    Necessary for pickling of dynamic class.
    """

    def __init__(self, wrapped):
        """
        :param wrapped: module for wrapping.
        :type wrapped: Module.
        """
        self.wrapped = wrapped

    def __dir__(self):
        return dir(self.wrapped)

    def __getattr__(self, name):
        """
        Override method `__getattr__` allows manipulate with modules.

        :param name: Name of specific object.
        :type name: string.
        :return: specific class when name of class starts with managed or
                 normal attribute from wrapped class.
        """
        if name not in self.wrapped.__dict__:
            if name.startswith("managed"):
                cls_name = name.split("_")
                cls = self.wrapped.__dict__[cls_name[1]]
                m_cls, _ = Manager(self.wrapped.__name__, self).factory(cls,
                                                                        _class_names=cls_name)
                return m_cls
        cls = getattr(self.wrapped, name)
        return cls


class VersionableClass(object):

    """
    Class used for marking of mutable class.
    """
    def __new__(cls, *args, **kargs):
        """
        If this method is invoked it means that something went wrong because
        this class should be replaced by :class:`Manager` factory.
        """
        raise Exception("Class %s is not prepared for usage. "
                        "You have to call versionable_class.factory(cls) "
                        "before you can use it" % (cls))


class Manager(object):

    def __init__(self, name, wrapper=None):
        """
        Manager for module.

        :param name: Name of module.
        :type name: string
        :param wrapper: Module dictionary wrapper. Should be None.
        """
        __import__(name)
        if not wrapper:
            if not isinstance(sys.modules[name], ModuleWrapper):
                self.wrapper = ModuleWrapper(sys.modules[name])
                sys.modules[name] = self.wrapper
            else:
                self.wrapper = sys.modules[name]
        else:
            self.wrapper = wrapper

    def factory(self, _class, *args, **kargs):
        """
        Create new class with right version of subclasses.

        Goes through class structure and search subclasses with right version.

        :param _class: Class which should be prepared.
        :type _class: class.
        :param args: Params for _is_right_ver function.
        :params kargs: Params for _is_right_ver function.
        """
        def add_to_structure(cl, new_bases):
            if VersionableClass in cl.__mro__:
                cls, cls_vn = self.factory(cl, *args, **kargs)
                new_bases.append(cls)
                return cls_vn
            else:
                new_bases.append(cl)
                return ""

        _class_names = None
        if "_class_names" in kargs:
            _class_names = kargs["_class_names"]
        if (_class.__name__.startswith("managed") and
                hasattr(_class, "__original_class__")):
            _class = _class.__original_class__
        new_bases = []
        cls_ver_name = ""
        if VersionableClass in _class.__bases__:  # parent is VersionableClass
            for m_cls in _class.__bases__:
                if m_cls is VersionableClass:
                    mro = _class.__master__.__mro__    # Find good version.
                    if _class_names:
                        for cl in mro[:-1]:
                            if cl.__name__ in _class_names:
                                cls_ver_name += "_" + cl.__name__
                                cls_ver_name += add_to_structure(cl, new_bases)
                                break
                    else:
                        for cl in mro[:-1]:
                            if cl._is_right_ver(*args, **kargs):
                                cls_ver_name += "_" + cl.__name__
                                cls_ver_name += add_to_structure(cl, new_bases)
                                break
                else:
                    cls_ver_name += add_to_structure(m_cls, new_bases)
        else:
            for m_cls in _class.__bases__:
                if (VersionableClass in m_cls.__mro__ or
                        hasattr(m_cls, "__original_class__")):
                    cls, cls_vn = self.factory(m_cls, *args, **kargs)
                    new_bases.append(cls)
                    cls_ver_name += cls_vn
                else:
                    new_bases.append(m_cls)
        class_name = "managed_%s%s" % (_class.__name__, cls_ver_name)

        if hasattr(self.wrapper.wrapped, class_name):
            # Don't override already created class.
            return self.wrapper.wrapped.__dict__[class_name], cls_ver_name

        class_dict = _class.__dict__.copy()
        class_dict["__original_class__"] = _class
        cls = type(class_name, tuple(new_bases), class_dict)
        self.wrapper.wrapped.__dict__[class_name] = cls

        return cls, cls_ver_name

    def __getitem__(self, o_cls):
        return self.getcls(*o_cls)

    def getcls(self, cls, orig_class):
        """
        Return class correspond class and original class.

        :param cls: class for which should be found derived alternative.
        :type cls: class
        :param orig_class: Original class
        :type orig_class: class

        :return: Derived alternative class
        :rtype: class
        """
        for m_cls in cls.__mro__:
            if hasattr(m_cls, "__original_class__"):
                if m_cls.__original_class__ is orig_class:
                    return m_cls
            elif m_cls is orig_class:
                return m_cls
        raise Exception("Couldn't find derived alternative in %s for"
                        " class %s" % (cls, orig_class))


def factory(orig_cls, *args, **kargs):
    """
    Create class with specific version.

    :param orig_class: Class from which should be derived good version.
    :param args: list of parameters for _ir_right_ver
    :params kargs: dict of named parameters for _ir_right_ver
    :return: params specific class.
    :rtype: class
    """
    return Manager(orig_cls.__module__).factory(orig_cls, *args, **kargs)[0]



"""
libvirt configuration related utility functions
"""

import logging
import re

from avocado.core import exceptions

from virttest import utils_config
from virttest import utils_libvirtd
from virttest import utils_split_daemons
from virttest import remote
from virttest.utils_test import libvirt


def remove_key_in_conf(value_list, conf_type="libvirtd",
                       remote_params=None, restart_libvirt=False):
    """
    Remove settings in configuration file on local or remote
    and restart libvirtd if needed

    :param value_list: A list of settings to delete, like ["log_level", "log_outputs"]
    :param conf_type: The configuration type to update on localhost,
        eg, "libvirt", "virtqemud"
    :param remote_params: The params for remote access
    :param restart_libvirt: True to restart libvirtd
    :return: remote.RemoteFile object for remote file or
        utils_config.LibvirtConfigCommon object for local configuration file
    """

    if remote_params:
        remote_ip = remote_params.get("server_ip",
                                      remote_params.get("remote_ip"))
        remote_pwd = remote_params.get("server_pwd",
                                       remote_params.get("remote_pwd"))
        remote_user = remote_params.get("server_user",
                                        remote_params.get("remote_user"))
        file_path = remote_params.get("file_path")
        if not all([remote_ip, remote_pwd, remote_user, file_path]):
            raise exceptions.TestError("remote_[ip|user|pwd] and file_path "
                                       "are necessary!")
        remote_file_obj = remote.RemoteFile(address=remote_ip, client='scp',
                                            username=remote_user,
                                            password=remote_pwd,
                                            port='22', remote_path=file_path)
        remote_file_obj.remove(value_list)

        if restart_libvirt:
            libvirt.remotely_control_libvirtd(remote_ip, remote_user,
                                              remote_pwd, action='restart',
                                              status_error='no')
        return remote_file_obj
    else:
        target_conf = utils_config.get_conf_obj(conf_type)
        for item in value_list:
            try:
                del target_conf[item]
            except utils_config.ConfigNoOptionError as err:
                logging.error(err)
        if restart_libvirt:
            libvirtd = utils_libvirtd.Libvirtd()
            libvirtd.restart()
        return target_conf


def remove_key_for_modular_daemon(params, remote_dargs=None):
    """
    Remove some configuration keys if the modular daemon is enabled.
    If you set "do_search" or/and "do_not_search" in params, it first
    checks the values and then removes the keys from the config file.

    :param param: The param to use
    :param remote_dargs: The params for remote access
    :return: remote.RemoteFile object for remote file or
        utils_config.LibvirtConfigCommon object for local configuration file
    """

    conf_obj = None
    session = None
    if remote_dargs:
        server_ip = remote_dargs.get("server_ip", remote_dargs.get("remote_ip"))
        server_pwd = remote_dargs.get("server_pwd", remote_dargs.get("remote_pwd"))
        server_user = remote_dargs.get("server_user", remote_dargs.get("remote_user"))
        if not all([server_ip, server_pwd, server_user]):
            raise exceptions.TestError("server_[ip|user|pwd] are necessary!")
        session = remote.wait_for_login('ssh', server_ip, '22', server_user,
                                        server_pwd, r"[\#\$]\s*$")

    if utils_split_daemons.is_modular_daemon(session):
        remove_key = eval(params.get("remove_key", "['remote_mode']"))
        conf_type = params.get("conf_type", "libvirt")
        search_cond = eval(params.get("do_search", '{}'))
        no_search_cond = eval(params.get("no_search", '{}'))
        for k, v in search_cond.items():
            if not re.search(v, k, re.IGNORECASE):
                logging.debug("The key '%s' does not contain '%s', "
                              "no need to remove %s in %s conf file.",
                              k, v, remove_key, conf_type)
                return
        for k, v in no_search_cond.items():
            if re.search(v, k, re.IGNORECASE):
                logging.debug("The key '%s' contains '%s', "
                              "no need to remove %s in %s conf file.",
                              k, v, remove_key, conf_type)
                return

        conf_obj = remove_key_in_conf(remove_key, conf_type=conf_type,
                                      remote_params=remote_dargs)
    if session:
        session.close()
    return conf_obj
